/* Program04_1
 *
 * "Yes" felirat kirsa msodpercenknt USART2 Tx kimenetn (PA2)
 *
 * Kssk a PA2 kivezetst egy USB-UART TTL talakt Rx lbra s
 * a Termite vagy ms terminl emultorban figyeljk a kirst!
 *
 * Alaprtelmezetten az APB1 busz frekvencija 36 MHz (fCPU/2).
 * Ennek figyelembe vtelvel konfigurljuk UART2 sebessgt 9600 Baudra.
 * Alaprtelmezetten a PA2 kivezetshez van az USART2 TX kimenet 
 * hozzrendelve 
 *
 * Felhasznlt forrs: 
 * Mazidi et al., STM32 Arm Programming for Embedded Systems
 * http://www.microdigitaled.com/ARM/STM_ARM_books.html
 *
 * Tested with Keil MDK-Lite v5.28 and STM32F1 DFP v2.3.0
 */
 
#include "stm32f10x.h"

void USART2_init(void);
void USART2_write(int c);
void delayMs(int n);

/*----------------------------------------------------------------------------
  MAIN function
 *----------------------------------------------------------------------------*/
int main (void) {
  USART2_init();                           // USART2 konfigurlsa

  while(1) { 
    USART2_write('Y');                     // Karakterek kirsa
    USART2_write('e');
    USART2_write('s');
    delayMs(1000);                         // Vrunk kt kirs kztt
  }
}

/*----------------------------------------------------------------------------
  UART2 s a PA2 kimenet inicializlsa
 *----------------------------------------------------------------------------*/
void USART2_init (void) {
  RCC->APB1ENR |= RCC_APB1ENR_USART2EN;    // USART2 engedlyezs
  RCC->APB2ENR |= RCC_APB2ENR_AFIOEN;      // AFIO rajel engedlyezs
  RCC->APB2ENR |= RCC_APB2ENR_IOPAEN;      // GPIOA rajel engedlyezs
  AFIO->MAPR &= ~AFIO_MAPR_USART2_REMAP;   // Trli USART2 REMAP bitjt 
  //--- PA2 konfigurlsa USART2 Tx digitlis kimenetknt -----------
  GPIOA->CRL &= ~(GPIO_CRL_MODE2|GPIO_CRL_CNF2); // PA2 CNF s Mode bitek trlse
  GPIOA->CRL |=  GPIO_CRL_MODE2_0 |        // PA2 pushpull alternate kimenet, max. 10 MHz
                 GPIO_CRL_CNF2_1;          // CNF: 10  MODE: 01
  //--- USART2 konfigurlsa: Tx only, 9600  8 N 1 mdba ------------
  USART2->BRR = 3750;                      // 9600 baud @ 72/2 MHz
  USART2->CR1 = USART_CR1_TE;              // Tx engedlyezs, 8-bit adat
  USART2->CR2 = 0x0000;                    // 1 stop bit
  USART2->CR3 = 0x0000;                    // nincs adatfolyam-vezrls
  USART2->CR1 |= USART_CR1_UE;             // USART2 engedlyezs
}

/*----------------------------------------------------------------------------
    Karakter kirsa az USART2 soros portra
 *----------------------------------------------------------------------------*/
void USART2_write (int ch) {
  while (!(USART2->SR & USART_SR_TXE));    // Vr, amg a Tx buffer fel nem szabadul
  USART2->DR = (ch & 0xFF);
}

/*----------------------------------------------------------------------------
    Ksleltet eljrs a SysTick felhasznlsval
 *----------------------------------------------------------------------------*/
void delayMs(int n) {
  int i;
  SysTick->LOAD = SystemCoreClock/1000-1;  // jratltsi rtk 1 ms ksleltetshez
  SysTick->VAL  = 0;                       // Szmll trlse
  SysTick->CTRL = 0x5;                     // Engedlyezs, nincs interrupt, rendszer rajel
  for(i = 0; i < n; i++) {
    while((SysTick->CTRL & 0x10000) == 0); // A COUNT jelzre vrunk
  }
  SysTick->CTRL = 0;                       // SysTick lelltsa
}




