/* Program04_2
 *
 * Az USART2 Rx bemenetn (PA3) fogadott szmjegy karakterrel
 *  vezreljk, hogy hnyszor villanjon fel a beptett LED (PC13).
 *
 * Kssk a PA3 kivezetst egy USB-UART TTL talakt Tx lbra,
 * s a Termite vagy ms terminl emultorral kapcsoldjunk a 
 * PC-n a mikrovezrlonkhz!
 *
 * Alaprtelmezetten az APB1 busz frekvencija 36 MHz (fCPU/2).
 * Az UART2 sebessgt 115200 Baudra konfigurljuk.
 * PA3 az alprtelmezett USART2 RX bemenet, ezt hasznljuk. 
 *
 * Felhasznlt forrs: 
 * Mazidi et al., STM32 Arm Programming for Embedded Systems
 * http://www.microdigitaled.com/ARM/STM_ARM_books.html
 *
 * Tested with Keil MDK-Lite v5.28 and STM32F1 DFP v2.3.0
 */
 
#include "stm32f10x.h"
void USART2_init(void);
int USART2_read(void);
void LED_blink(int value);
void delayMs(int n);

/*----------------------------------------------------------------------------
  MAIN function
 *----------------------------------------------------------------------------*/
int main(void) {
  char c;
  RCC->APB2ENR |=  RCC_APB2ENR_IOPCEN;       // GPIOC rajel engedlyezse
  GPIOC->CRH &= ~(GPIO_CRH_CNF13 | GPIO_CRH_MODE13); // pin13 CNF s Mode bitek trlse  
  GPIOC->CRH |= GPIO_CRH_MODE13_1;           // PC13 CNF: 00 Mode: 10 bellts
                                             // (GPIO puhpull output, max. 2 MHz)   	
  USART2_init();
  while (1) {
    c = USART2_read();                       // Berkez karakterre vrunk
    LED_blink(c);                            // LED villogtats
  }
}

/*----------------------------------------------------------------------------
  UART2 s a PA3 bemenet inicializlsa
 *----------------------------------------------------------------------------*/
void USART2_init (void) {
  RCC->APB1ENR |= RCC_APB1ENR_USART2EN;      // USART2 engedlyezs
  RCC->APB2ENR |= RCC_APB2ENR_AFIOEN;        // AFIO rajel engedlyezs
  RCC->APB2ENR |= RCC_APB2ENR_IOPAEN;        // GPIOA rajel engedlyezs
  AFIO->MAPR &= ~AFIO_MAPR_USART2_REMAP;     // Trli USART2 REMAP bitjt 
  //--- PA3 konfigurlsa digitlis bemenetknt  -----------
  GPIOA->CRL &= ~(GPIO_CRL_MODE3|GPIO_CRL_CNF3); // PA2 CNF s Mode bitek trlse
  GPIOA->CRL |=  GPIO_CRL_CNF3_0;            // PA3 digitlis bemenet CNF: 01  MODE: 00
  //--- USART2 konfigurlsa: Rx only, 115200,  8 N 1 mdba ------------
  USART2->BRR = 312;                         // 115200 baud @ 72/2 MHz
  USART2->CR1 = USART_CR1_RE;                // Rx engedlyezs, 8-bit adat
  USART2->CR2 = 0x0000;                      // 1 stop bit
  USART2->CR3 = 0x0000;                      // nincs adatfolyam-vezrls
  USART2->CR1 |= USART_CR1_UE;               // USART2 engedlyezs
}

/*----------------------------------------------------------------------------
  Read a character from USART2 
 *----------------------------------------------------------------------------*/
int USART2_read(void) {
  while (!(USART2->SR & USART_SR_RXNE));     // vr, amg egy karakter nem rkezik
  return USART2->DR;
}

/* turn on or off the LEDs according to the value */
void LED_blink(int value) {
  value = value - 48;
  if(value>0 && value <10) {                 // Csak 1 - 9 lehet
    for (; value > 0; value--) {
      GPIOC->BSRR = GPIO_BSRR_BR13;          // GPIOC 13. bit trls (LED be) 
      delayMs(200);
      GPIOC->BSRR = GPIO_BSRR_BS13;          // GPIOC 13. bit bellts (LED ki)
      delayMs(200);
    }
    delayMs(800);
  }
}

/*----------------------------------------------------------------------------
    Ksleltet eljrs a SysTick felhasznlsval
 *----------------------------------------------------------------------------*/
void delayMs(int n) {
  int i;
  SysTick->LOAD = SystemCoreClock/1000-1;    // jratltsi rtk 1 ms ksleltetshez
  SysTick->VAL  = 0;                         // Szmll trlse
  SysTick->CTRL = 0x5;                       // Engedlyezs, nincs interrupt, rendszer rajel
  for(i = 0; i < n; i++) {
    while((SysTick->CTRL & 0x10000) == 0);   // A COUNT jelzre vrunk
  }
  SysTick->CTRL = 0;                         // SysTick lelltsa
}

