/* Program04_3
 *
 * "Hello" zenet kirsa msodpercenknt USART1 Tx kimenetn (PB6)
 *
 * Kssk a PB6 kivezetst egy USB-UART TTL talakt Rx lbra s
 * a Termite vagy ms terminl emultorban figyeljk a kirst!
 *
 * Alaprtelmezetten az APB2 busz frekvencija 72 MHz (fCPU).
 * Ennek figyelembe vtelvel konfigurljuk UART1 sebessgt 9600 Baudra.
 * Alaprtelmezetten a PA9 kivezetshez van az USART1 TX kimenet 
 * hozzrendelve, de itt az alternatv hozzrendelst (PB6) 
 * hasznljuk. Ehhez az AFIO->MAPR regiszter 2. bitjt 
 * (AFIO_MAPR_USART1_REMAP) 1-be kell lltani. 
 *
 * Felhasznlt forrs: 
 * Mazidi et al., STM32 Arm Programming for Embedded Systems
 * http://www.microdigitaled.com/ARM/STM_ARM_books.html
 *
 * Tested with Keil MDK-Lite v5.28 and STM32F1 DFP v2.3.0
 */
 
#include "stm32f10x.h"

void USART1_init(void);
void USART1_write(int c);
void delayMs(int n);

/*----------------------------------------------------------------------------
  MAIN function
 *----------------------------------------------------------------------------*/
int main (void) {
  char message[] = "Hello\r\n";
  USART1_init();                             // USART1 konfigurlsa

  while(1) {  
    for (int i = 0; i < 7; i++) {            // a string kiratsa 
      USART1_write(message[i]);
    }
    delayMs(1000);                           // Kt kirats kztt vrunk
  }
}

/*----------------------------------------------------------------------------
  USART1 s a PB6 kimenet inicializlsa
 *----------------------------------------------------------------------------*/
void USART1_init (void) {
  RCC->APB2ENR |= RCC_APB2ENR_USART1EN;      // USART1 engedlyezs
  RCC->APB2ENR |= RCC_APB2ENR_AFIOEN;        // AFIO rajel engedlyezs
  RCC->APB2ENR |= RCC_APB2ENR_IOPBEN;        // GPIOB rajel engedlyezs
  AFIO->MAPR |= AFIO_MAPR_USART1_REMAP;      // Belltja USART1 REMAP bitjt 
  //--- PB6 konfigurlsa USART1 Tx digitlis kimenetknt -----------
  GPIOB->CRL &= ~(GPIO_CRL_MODE6|GPIO_CRL_CNF6); // PB6 CNF s Mode bitek trlse
  GPIOB->CRL |=  GPIO_CRL_MODE6_0 |          // PA2 pushpull alternate kimenet, max. 10 MHz
                 GPIO_CRL_CNF6_1;            // CNF: 10  MODE: 01
  //--- USART1 konfigurlsa: Tx only, 9600  8 N 1 mdba ------------
  USART1->BRR = 7500;                        // 9600 baud @ 72 MHz (APB2!)
  USART1->CR1 = USART_CR1_TE;                // Tx engedlyezs, 8-bit adat
  USART1->CR2 = 0x0000;                      // 1 stop bit
  USART1->CR3 = 0x0000;                      // nincs adatfolyam-vezrls
  USART1->CR1 |= USART_CR1_UE;               // USART1 engedlyezs
}

/*----------------------------------------------------------------------------
  Egy karakter kirsa USART1-re
 *----------------------------------------------------------------------------*/
void USART1_write (int ch) {
  while (!(USART1->SR & USART_SR_TXE));      // Vr, amg a Tx buffer fel nem szabadul
  USART1->DR = (ch & 0xFF);
}

/*----------------------------------------------------------------------------
    Ksleltet eljrs a SysTick felhasznlsval
 *----------------------------------------------------------------------------*/
void delayMs(int n) {
  int i;
  SysTick->LOAD = SystemCoreClock/1000-1;    // jratltsi rtk 1 ms ksleltetshez
  SysTick->VAL  = 0;                         // Szmll trlse
  SysTick->CTRL = 0x5;                       // Engedlyezs, nincs interrupt, rendszer rajel
  for(i = 0; i < n; i++) {
    while((SysTick->CTRL & 0x10000) == 0);   // A COUNT jelzre vrunk
  }
  SysTick->CTRL = 0;                         // SysTick lelltsa
}




