/* Program04_5
 *
 * A C stdio knyvtr hasznlatnak bemutatsa. A fputc() s fgetc() 
 * konzol I/O fggvnyeket az USART2_write() s USART2_read()
 * fggvnyekkel valstjuk meg.
 *
 * Kssk a PA2 kivezetst egy USB-UART TTL talakt Rx lbra,
 * a PA3 kivezetst pedig a Tx lbra s a Termite vagy ms terminl 
 * emultorral kapcsoldjunk a PC-n a mikrovezrlonkhz!
 *
 * Alaprtelmezetten az APB1 busz frekvencija 36 MHz (fCPU/2).
 * Ennek figyelembe vtelvel konfigurljuk UART2 sebessgt 9600 Baudra.
 * Alaprtelmezetten az USART2 TX kimenet a PA2 kivezetshez van 
 * hozzrendelve, az USART2 RX bemenet pedig PA3-hoz. 
 *
 * Felhasznlt forrs: 
 * Mazidi et al., STM32 Arm Programming for Embedded Systems
 * http://www.microdigitaled.com/ARM/STM_ARM_books.html
 *
 * Tested with Keil MDK-Lite v5.28 and STM32F1 DFP v2.3.0
 */
 
#include "stm32f10x.h"
#include <stdio.h>
void USART2_init(void);
int USART2_write(int c);
int USART2_read(void);
void delayMs(int n);

/*----------------------------------------------------------------------------
  MAIN function
 *----------------------------------------------------------------------------*/
int main(void) {
  int n;
  char str[80];
  USART2_init();
  printf("Test stdio library console I/O functions\r\n");
  fprintf(stdout, "    test for stdout\r\n");
  fprintf(stderr, "    test for stderr\r\n");

  while (1) {
    printf("please enter a number: ");
    scanf("%d", &n);
    printf("the number entered is: %d\r\n", n);
    printf("please type a character string: ");
    gets(str);
    printf("the character string entered is: ");
    puts(str);
    printf("\r\n");
  }
}

/*----------------------------------------------------------------------------
  UART2 s a PA2/PA3 kivezetsek inicializlsa
 *----------------------------------------------------------------------------*/
void USART2_init (void) {
  RCC->APB1ENR |= RCC_APB1ENR_USART2EN;      // USART2 engedlyezs
  RCC->APB2ENR |= RCC_APB2ENR_AFIOEN;        // AFIO rajel engedlyezs
  RCC->APB2ENR |= RCC_APB2ENR_IOPAEN;        // GPIOA rajel engedlyezs
  AFIO->MAPR &= ~AFIO_MAPR_USART2_REMAP;     // Trli USART2 REMAP bitjt 
  //--- PA2 konfigurlsa USART2 Tx digitlis kimenetknt -----------
  GPIOA->CRL &= ~(GPIO_CRL_MODE2|GPIO_CRL_CNF2); // PA2 CNF s Mode bitek trlse
  GPIOA->CRL |=  GPIO_CRL_MODE2_0 |          // PA2 pushpull alternate kimenet, max. 10 MHz
                 GPIO_CRL_CNF2_1;            // CNF: 10  MODE: 01
  //--- PA3 konfigurlsa digitlis bemenetknt  -----------
  GPIOA->CRL &= ~(GPIO_CRL_MODE3|GPIO_CRL_CNF3); // PA2 CNF s Mode bitek trlse
  GPIOA->CRL |=  GPIO_CRL_CNF3_0;            // PA3 digitlis bemenet CNF: 01  MODE: 00
  //--- USART2 konfigurlsa: Tx only, 9600  8 N 1 mdba ------------
  USART2->BRR = 3750;                        // 9600 baud @ 72/2 MHz
  USART2->CR1 = USART_CR1_TE | USART_CR1_RE; // Rx, Tx engedlyezs, 8-bit adat
  USART2->CR2 = 0x0000;                      // 1 stop bit
  USART2->CR3 = 0x0000;                      // nincs adatfolyam-vezrls
  USART2->CR1 |= USART_CR1_UE;               // USART2 engedlyezs
}

/*----------------------------------------------------------------------------
  Write a character to USART2
 *----------------------------------------------------------------------------*/
int USART2_write (int ch) {
  while (!(USART2->SR & 0x0080)) {}          // wait until Tx buffer empty
  USART2->DR = (ch & 0xFF);
  return ch;
}

/*----------------------------------------------------------------------------
  Read a character from USART2 
 *----------------------------------------------------------------------------*/
int USART2_read(void) {
  while (!(USART2->SR & USART_SR_RXNE));     // vr, amg egy karakter nem rkezik
  return USART2->DR;
}

/*----------------------------------------------------------------------------
    Ksleltet eljrs a SysTick felhasznlsval
 *----------------------------------------------------------------------------*/
void delayMs(int n) {
  int i;
  SysTick->LOAD = SystemCoreClock/1000-1;    // jratltsi rtk 1 ms ksleltetshez
  SysTick->VAL  = 0;                         // Szmll trlse
  SysTick->CTRL = 0x5;                       // Engedlyezs, nincs interrupt, rendszer rajel
  for(i = 0; i < n; i++) {
    while((SysTick->CTRL & 0x10000) == 0);   // A COUNT jelzre vrunk
  }
  SysTick->CTRL = 0;                         // SysTick lelltsa
}

/*----------------------------------------------------------------------------
 * Az albbi kd az interfesz a C standard I/O libraryhoz.
 * Minden I/O a konzolra van tirnytva, ami most az USART2.
 *----------------------------------------------------------------------------*/
struct __FILE { int handle; };
FILE __stdin  = {0};
FILE __stdout = {1};
FILE __stderr = {2};

/* A C library console/file beolvass hvja meg
 * Visszatkrzi a vett karaktert.
 * Ha a beolvasott karakter '\r', akkor '\n'-nel helyettestjk.
 */
int fgetc(FILE *f) {
  int c;
  c = USART2_read();                         // Karakter beolvassa a konzolrl
  if (c == '\r') {                           // Ha '\r', akkor kirs utn '\n'-nel helyettestjk
    USART2_write(c);                         // Visszatkrzs
    c = '\n';
  }
  USART2_write(c);                           // Visszatkrzs
  return c;
}

/* A C library console/file kirats hvja meg */
int fputc(int c, FILE *f) {
  return USART2_write(c);                    // A karakter kiratsa
}


