/* Program04_4
 *
 * Az USART1 Rx bemenetn (PB7) fogadott karaktert
 * visszarjuk az USART1 Tx kimeneten (PB6).
 *
 * Kssk a PB6 kivezetst egy USB-UART TTL talakt Rx lbra,
 * a PB7 kivezetst pedig a Tx lbra s a Termite vagy ms terminl 
 * emultorral kapcsoldjunk a PC-n a mikrovezrlonkhz!
 *
 * Alaprtelmezetten az APB2 busz frekvencija 72 MHz (fCPU).
 * Ennek figyelembe vtelvel konfigurljuk UART1 sebessgt 9600 Baudra.
 * Alaprtelmezetten a PA9/PA10 kivezetsekhez vannak rendelve az USART1
 * TX/RX kivezetsei, de most az alternatv kiosztst (RB6/RB7) hasznljuk.
 * Ehhez az AFIO->MAPR regiszter 2. bitjt AFIO_MAPR_USART1_REMAP) 
 * 1-be kell lltani. 
 *
 * Felhasznlt forrs: 
 * Mazidi et al., STM32 Arm Programming for Embedded Systems
 * http://www.microdigitaled.com/ARM/STM_ARM_books.html
 *
 * Tested with Keil MDK-Lite v5.28 and STM32F1 DFP v2.3.0
 */
 
#include "stm32f10x.h"

void USART1_init(void);
void USART1_write(char ch);
int USART1_read(void);
void outString(char* p);
void delayMs(int n);

/*----------------------------------------------------------------------------
  MAIN function
 *----------------------------------------------------------------------------*/
int main(void) {
  char c; 	
  USART1_init();
  while (1) {
    c = USART1_read();                       // Karakter berkezsre vrunk
    outString("\r\nVett karakter: ");        // Egy fix szveg kiratsa
    USART1_write(c);                         // A vett karakter kiratsa
  }
}

/*----------------------------------------------------------------------------
  USART1 s a PB6/PB7 kivezetsek inicializlsa
 *----------------------------------------------------------------------------*/
void USART1_init (void) {
  RCC->APB2ENR |= RCC_APB2ENR_USART1EN;      // USART1 engedlyezse
  RCC->APB2ENR |= RCC_APB2ENR_AFIOEN;        // AFIO rajel engedlyezs
  RCC->APB2ENR |= RCC_APB2ENR_IOPBEN;        // GPIOB rajel engedlyezs
  AFIO->MAPR |= AFIO_MAPR_USART1_REMAP;      // Belltja USART1 REMAP bitjt 
  //--- PB6 konfigurlsa USART1 Tx digitlis kimenetknt -----------
  GPIOB->CRL &= ~(GPIO_CRL_MODE6|GPIO_CRL_CNF6); // PB6 CNF s Mode bitek trlse
  GPIOB->CRL |=  GPIO_CRL_MODE6_0 |          // PA2 pushpull alternate kimenet, max. 10 MHz
                 GPIO_CRL_CNF6_1;            // CNF: 10  MODE: 01
  //--- PB7 konfigurlsa digitlis bemenetknt  -----------
  GPIOB->CRL &= ~(GPIO_CRL_MODE7|GPIO_CRL_CNF7); // PB7 CNF s Mode bitek trlse
  GPIOB->CRL |=  GPIO_CRL_CNF7_0;            // PB7 digitlis bemenet CNF: 01  MODE: 00	

  //--- USART1 konfigurlsa: Tx&Rx, 9600  8 N 1 mdba --------------
  USART1->BRR = 7500;                        // 9600 baud @ 72 MHz (APB2!)
  USART1->CR1 = USART_CR1_TE | USART_CR1_RE; // Tx, Rx engedlyezs, 8-bit adat
  USART1->CR2 = 0x0000;                      // 1 stop bit
  USART1->CR3 = 0x0000;                      // nincs adatfolyam-vezrls
  USART1->CR1 |= USART_CR1_UE;               // USART1 engedlyezs
}

/*----------------------------------------------------------------------------
  Egy karakter beolvassa USART1-rl 
 *----------------------------------------------------------------------------*/
int USART1_read(void) {
  while (!(USART1->SR & USART_SR_RXNE));     // vr, amg egy karakter nem rkezik
  return USART1->DR;
}

/*----------------------------------------------------------------------------
  Egy karakter kirsa USART1-re
 *----------------------------------------------------------------------------*/
void USART1_write (char ch) {
  while (!(USART1->SR & USART_SR_TXE));      // Vr, amg a Tx buffer fel nem szabadul
  USART1->DR = (ch & 0xFF);
}

/*----------------------------------------------------------------------------
  Kir egy nullval lezrt szveget az USART1 porton
 *----------------------------------------------------------------------------*/
void outString(char* psz_s) {
  char c;
  while ((c = *psz_s)) {
      USART1_write(c);
      psz_s++;
  }
}

/*----------------------------------------------------------------------------
    Ksleltet eljrs a SysTick felhasznlsval
 *----------------------------------------------------------------------------*/
void delayMs(int n) {
  int i;
  SysTick->LOAD = SystemCoreClock/1000-1;    // jratltsi rtk 1 ms ksleltetshez
  SysTick->VAL  = 0;                         // Szmll trlse
  SysTick->CTRL = 0x5;                       // Engedlyezs, nincs interrupt, rendszer rajel
  for(i = 0; i < n; i++) {
    while((SysTick->CTRL & 0x10000) == 0);   // A COUNT jelzre vrunk
  }
  SysTick->CTRL = 0;                         // SysTick lelltsa
}

